#!/bin/sh

########################################################################
######################### GLOBAL DEFENITIONS ###########################
########################################################################

DIG_TOOL='/usr/sbin/dig'                                         # DIG tool placment
DNSLIST="64.87.26.147 24.249.204.58"                             # DNS base list
DNS_PROTO='+tcp'                                                 # DNS protocol to use - tcp
DNS_PROTO_ALT='+notcp'                                           # DNS protocol to use - udp
DNS_PORT=53                                                      # DNS port to use
DNS_PORT_BASE=53                                                 # DNS base port
DNS_PORT_ALT=5353                                                # DNS alternate port
DNS_DOMAIN_BASE='dns-servers.threatstop.local'                   # DNS base domain
DNS_DOMAIN_ALT='dns-servers.threatstop.com'                      # DNS alternate domain
DNS_SERVERS_ALT="8.8.8.8 8.8.4.4 208.67.220.220 208.67.222.222"  # DNS global servers list
DNS_SERVERS_LCL='127.0.0.1'                                      # DNS local defined servers list
DNS_MAX_TRIES=3                                                  # Number of tries before abort
DNS_NEW_LIST=''                                                  # New DNS list returned by query

[ ! -z "$CONFIG" ] && CONFIG="threatstop.conf"
CONFIG_TMP="threatstop.conf.tmp"

MANDATORY_PARAM_LST="DNSLIST BLOCKLIST ALLOWLIST DEVICE_IP"

#DNSLIST="24.249.204.58 64.87.26.147"
#DNS_SERVERS_ALT="8.8.8.9"  

#DEBUG=true

########################################################################
##################### FUNCTIONS DEFENITIONS ############################
########################################################################

##########################################################
# echoMsg : defines output type                          #
##########################################################
echoMsg() {

        case $1 in
                ERROR) echo "[ERROR] : $2"
                        ;;
                WARN)  echo "[WARN ] : $2"
                        ;;
                ALERT) echo "[ALERT] : $2"
                        ;;
                INFO)  echo "[INFO ] : $2"
                        ;;
                DEBUG) if [ "$DEBUG" = true ]; then
                                echo -e "\t[DEBUG] : $2"
                       fi
                        ;;
                *) echo "SCRIPT ERROR : Unrecognized parameter [$1]"
        esac
}

###########################################################
# FNgetCMDparams : reads CMD supplied data                #
###########################################################
FNgetCMDparams() {

	echoMsg INFO "Reading CMD supplied data"	
	
	while [ ! -z "$1" ]; do	
		case $1 in 
			-b|--blocklist)BLOCKLIST=$2
				shift
				;;
			-a|--allowlist)ALLOWLIST=$2
				shift
				;;
			-i|--deviceip)DEVICE_IP=$2
				shift
				;;
			-l|--logicalsystem)LS_NAME=$2
				shift
				;;
			*)echoMsg ERROR "Unrecognized parameter [$1]"
			  return 1
				;;
		esac
		shift
	done

	return 0
}

############################################################
# FNcreateConfFile : creates basic configuration file      #
#                     $CONFIG     = threatstop.conf        #
############################################################
FNcreateConfFile() {

	echoMsg INFO "Creating basic configuration [$CONFIG] file"
	echo "# Configuration file for the ThreatSTOP Juniper scripts" > $CONFIG
	echo >> $CONFIG
	echo "VERSION=\"$VERSION\"" >> $CONFIG
	echo "NDIG=\"/usr/sbin/dig +tcp\"" >> $CONFIG
	echo "DNSLIST=\"$DNSLIST\"" >> $CONFIG
	echo  >> $CONFIG

	return 0
}

##########################################################
#FNloadConfig: load data from configuration script       #
#              CONFIG="threatstop.conf"                  #
##########################################################
FNloadConfig() {

        echoMsg INFO "Loading configuration file [$CONFIG]"

        if [ ! -f $CONFIG ]; then
                echoMsg WARN "Cannot find config file... going to re-create it from scratch"
                FNcreateConfFile
		[ $? -ne 0 ] &&  return 1
        fi

	#Now load settings
        . ./$CONFIG

	return 0
}

############################################################
# FNverifySettings : verify for missing mandatory settings #
#                     $CONFIG     = threatstop.conf        #
#                     $CONFIG_TMP = threatstop.conf.tmp    #
############################################################
FNverifySettings() {

        #For next check we need to emulate array functionality missing in Bourny shell
        echoMsg INFO "Verifying loaded settings"

        local isOK=1

        for item in $MANDATORY_PARAM_LST; do 
                param=$(eval echo \$$item)
                if [ -z "$param" ]; then
                        echoMsg ERROR "Missing mandatory parameter - $item"
                        isOK=0
                fi  
        done    

        [ $isOK -eq 0 ] && return 1

	#Clean up settings we will reload later                        
        cat $CONFIG | sed -e '/ADDRESS_BOOK/d' -e '/CLUSTER/d' -e '/TRUST/d' > $CONFIG_TMP
        mv $CONFIG_TMP $CONFIG
                
        #Unset values we are going to re-detect anyway... old settings can mess our flow 
        #if we are running setup over previous installation
        ADDRESS_BOOK_TYPE=''
        ADDRESS_BOOK_TRUST=''
        ADDRESS_BOOK_UNTRUST=''
        CLUSTER=''
        UNTRUST=''
        TRUST=''         
                        
        return 0
}

##############################################################################
# FNupdateDNSConfFile : Update threatstop.conf file with new settings        #
#                     $CONFIG     = threatstop.conf                          #
#                     $CONFIG_TMP = threatstop.conf.tmp                      #
##############################################################################
FNupdateConfFile() {

        echoMsg INFO "Updating config file with new CMD supplied settings"

        if [ ! -z "$ALLOWLIST" ]; then
               	echoMsg DEBUG "Updating ALLOW LIST parameter with  [$ALLOWLIST]"
		if cat $CONFIG | grep 'ALLOWLIST=' 1>/dev/null 2>/dev/null; then
                	cat $CONFIG | sed "s/ALLOWLIST=.*/ALLOWLIST=\"$ALLOWLIST\"/g" > $CONFIG_TMP
			mv $CONFIG_TMP $CONFIG
		else
			echo "ALLOWLIST=\"$ALLOWLIST\"" >> $CONFIG
		fi
        fi

        if [ ! -z "$BLOCKLIST" ]; then
               	echoMsg DEBUG "Updating BLOCK LIST parameter with  [$BLOCKLIST]"
		if cat $CONFIG | grep 'BLOCKLIST=' 1>/dev/null 2>/dev/null; then
                        cat $CONFIG | sed "s/BLOCKLIST=.*/BLOCKLIST=\"$BLOCKLIST\"/g" > $CONFIG_TMP
                        mv $CONFIG_TMP $CONFIG
                else
                        echo "BLOCKLIST=\"$BLOCKLIST\"" >> $CONFIG
                fi
	fi

	if [ ! -z "$DEVICE_IP" ]; then
		echoMsg DEBUG "Updating DEVICE IP parameter with  [$DEVICE_IP]"
                if cat $CONFIG | grep 'DEVICE_IP=' 1>/dev/null 2>/dev/null; then
                        cat $CONFIG | sed "s/DEVICE_IP=.*/DEVICE_IP=\"$DEVICE_IP\"/g" > $CONFIG_TMP
                        mv $CONFIG_TMP $CONFIG
                else
                        echo "DEVICE_IP=\"$DEVICE_IP\"" >> $CONFIG
                fi
        fi

	#Update logic system name setting
	if [ ! -z "$LOGICSYS" ]; then
		echoMsg DEBUG "Updating LOGIC SYSTEM parameter with  [$LOGICSYS]"
                if cat $CONFIG | grep 'LOGICSYS=' 1>/dev/null 2>/dev/null; then
                        cat $CONFIG | sed "s/LOGICSYS=.*/LOGICSYS=\"$LOGICSYS\"/g" > $CONFIG_TMP
                        mv $CONFIG_TMP $CONFIG
                else
                        echo "BLOCKLIST=\"$BLOCKLIST\"" >> $CONFIG
                fi
        fi

	return 0
}


###########################################################
# aborting : abort flow execution                         #
###########################################################
aborting() {

        echo
        echo "************************************"
        echo "* Installation aborted by user ... *"
        echo "**i*********************************"
        echo

        exit 1
}

###########################################################
# exitting : exit flow                                    #
###########################################################
exitting() {

        echo
        echo "****************************************************************"
        echo "*Unrecoverable errors enconted ... aborting...                 *"
        echo "*Please fix then run script again                              *"
        echo "****************************************************************"
        echo

        exit 1
}

#########################################################
# setLock : Locks execution flow                        #
#########################################################
setLock() {

        trap "/bin/rm -f $LOCKFILE; return 1" 0 1 2 3 15

        echoMsg INFO "Locking current instance"

        if [ -e $LOCKFILE ]; then
                ps=$(cat $LOCKFILE)
                echoMsg ERROR "Failed to lock ... Another instance [$ps] of the same script is runnng"
                return 1
        fi

        echo $$ > $LOCKFILE
        return 0
}

#########################################################
# unLock : Unlocks execution flow                       #
#########################################################
unLock() {

        echoMsg INFO "Unlocking current instance"
        /bin/rm -f $LOCKFILE
        return 0
}

#########################################################
# verifyUserID : Verify user ID ( super user ? )        #
#########################################################
verifyUserID() {

        #Should be 0 otherwise abort
        if [ $MYUID -ne 0 ]; then
                echoMsg ERROR "Wrong UID... must be run under superuser credentials..."
                return 1
        fi

        return 0
}

################################################################################
# getDNSserversList : keep result in tmp file
#                     $ip {
#                         res_time   = $dns_time
#                         res_status = $dns_status
#                     }
################################################################################
getDNSserversList() {

	local dns_tool=$1
	local dns_to_ask=$2
	local dns_to_res=$3
	local dns_proto=$4
	local dns_port=$5
	local dns_qtype=$6

	local rv
	local rc
	local ip
	local dns_res

	#Run dig quiery against $dns_to_ask server resolving $dns_to_res domain name
	echoMsg INFO "Querying  DNS $dns_to_ask server for domain $dns_to_res"
	echoMsg DEBUG "$DIG_TOOL $dns_proto -p $dns_port \@$dns_to_ask $dns_to_res"
	
	rv=$($DIG_TOOL $dns_proto -p $dns_port @$dns_to_ask $dns_to_res 2>&1)
	rc=$?

	#####################################################
	#Ready to parse returned data. We are intrested in  #
	# - DNS serves list                                 #
	# - Query status                                    #
	# - Response time                                   #
	#####################################################

	dns_res=$(echo $rv | awk 'BEGIN { RS = ";" } 
					/opcode: QUERY, status:/ {
						for (i=1; i<=NF; i++) {
							if ( $i ~ /status:/ ) {
								gsub(",","");
								print "export dns_status="$(i+1);
								break;
							} 
						}
				      	} 
                                      	/ANSWER SECTION:/ { 
						printf("export dns_list_res=\"");
                                           	for (i=1; i<=NF; i++) { 
                                               		if ( $i ~ /[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/ ) { 
                                                  		printf("%s ",$i); 
                                               		}
                                           	}
						printf("\"\n");
                                      	}
				      	/Query time:/ {
						printf("export dns_time=")
						for(i=1; i<=NF; i++) {
							if ( $i ~ /[0-9]+/ ) {
								print $i;
								break;
							}
						}
				      	}	
                                 '
                ) 

	eval $dns_res

	#if DNS query did not return NOERROR status. Asume this call is a failure
	if [ "$dns_status" != "NOERROR" ]; then
		echoMsg DEBUG "DNS query to DNS server [$dns_to_ask] returned with bad status"
		return 2
	fi

	#Build new returned DNS list to compare with previous list
	if [ -z "$DNS_NEW_LIST" ]; then
		DNS_NEW_LIST="$dns_list_res"
	else 
		for ip in $dns_list_res; do
			if ! echo "$DNS_NEW_LIST" | grep -q "$ip"; then
				DNS_NEW_LIST="$DNS_NEW_LIST $ip"
			fi
		done
	fi
	
	#Build struct holding ip relevent data. But first convert dots to underscore
	ip=$(echo $dns_to_ask | sed 's/\./_/g')
	eval res_${ip}_time="$dns_time"
	eval res_${ip}_status="$dns_status"
	

	return 0
}

##############################################################################
# readLocalBoxNameserversLst : read list DNS servers defined on local box    #
##############################################################################
readLocalBoxNameserversLst() {

	echoMsg INFO "Getting local defined DNS servers list"

	DNS_SERVERS_LCL=$(cat /etc/resolv.conf | grep nameserver | /usr/bin/awk '{print $2}')

	[ -z "$DNS_SERVERS_LCL" ] && return 1
	return 0
}

##############################################################################
# updateDNSConfFile : Update threatstop.conf file with new settings          #
##############################################################################
updateDNSConfFile() {

	echoMsg INFO "Updating config file with new DNS settings"

	if [ ! -z "$DNS_NEW_LIST" ]; then
		echoMsg DEBUG "Updating DNS list with settings [$DNS_NEW_LIST]"
		cat threatstop.conf | sed "s/DNSLIST=.*/DNSLIST=\"$DNS_NEW_LIST\"/g" > threatstop.conf.tmp
	else
		echoMsg DEBUG "Keeping previous DNS list version"
		cat threatstop.conf | sed "s/DNSLIST=.*/DNSLIST=\"$DNSLIST\"/g" > threatstop.conf.tmp
	fi

	#if cat threatstop.conf | grep 'PORT=' 1>/dev/null 2>/dev/null; then
	#	echoMsg DEBUG "Updating PORT setting with value [$DNS_PORT]"
	#	cat threatstop.conf.tmp | sed "s/PORT=.*/PORT=\"$DNS_PORT\"/g" > threatstop.conf
	#	rm -f threatstop.conf.tmp
	#else
	#	echoMsg DEBUG "Adding PORT setting with value [$DNS_PORT]"
	#	echo "PORT=\"$DNS_PORT\"" >> threatstop.conf.tmp
	#	mv threatstop.conf.tmp threatstop.conf
	#fi

	return 0
}

##############################################################################
# setNewDNSserversList :
##############################################################################
setNewDNSserversList() {

	local ip
	local res_time
	local res_time_qst=0
	local res_ip_qst

	echoMsg INFO "Detecting quickest to respond server"

	for ip in $DNSLIST; do
		
		#Change dots to _ and do redirect reference access to res_${ip}_time parameter
		ip=$(echo $ip | sed 's/\./\_/g')
		res_time=$(eval echo \$res_${ip}_time)

		#If this ip failed to respond than we do not have time setting for this ip
		#set it to some 10000 value to make sure it is not choosen
		[ -z "$res_time" ] && res_time=10000
	
		#This is the first parameter
		if [ $res_time_qst = 0 ]; then
			res_ip_qst=$ip
			res_time_qst=$res_time
			continue
		fi

		#Have found new quickest to respond server
		if [ $res_time -lt $res_time_qst ]; then
			res_ip_qst=$ip
                        res_time_qst=$res_time
		fi
	
		#The res time is slower than already found, move to next values
		
	done

	#Now re-build DNS servers list while puting the quickest to respond ip to the start
	#of the list
	res_ip_qst=$(echo $res_ip_qst | sed 's/\_/\./g')
	DNSLIST=$(echo $DNSLIST | sed "s/$res_ip_qst//")
	DNSLIST="$res_ip_qst $DNSLIST"
	
	return 0
}

##############################################################################
# compareDNSlists : compare two DNS lists                                    #
#	           return 0 - if lists are similar                           #
#                  return 1 - if lists are different                         # 
##############################################################################
compareDNSlists() {

	local dns_list_old="$1"
	local dns_list_new="$DNS_NEW_LIST"

	#DNS lists can not be empty
	[ -z "$dns_list_old" ] && echoMsg ERROR "Get empty DNS list" && return 2
	[ -z "$dns_list_new" ] && echoMsg ERROR "Get empty DNS list" && return 2

	echoMsg INFO "Comparing DNS lists for differences"
	dns_list_old=$(for i in $dns_list_old; do echo $i; done | sort)
	dns_list_new=$(for i in $dns_list_new; do echo $i; done | sort)
	
	dns_list_old=$(echo $dns_list_old)
	dns_list_new=$(echo $dns_list_new)

	if [ "$dns_list_old" = "$dns_list_new" ]; then
		#Lists are similar
		return 0
	else
		#Lists are different
		return 1
	fi
}

##############################################################################
# updateDNSlistsDo :                                                         #
#                    dig_tool    - dig tool plavment                         #
#                    dns_lists   - list of DNS servers ip's to run check on  #
#                    dns_domain  - server full name to resolve               #
#                    dns_proto   - protocol to use tcp/udp                   #
#                    dns_res_lst - DNS list created by our quieries          #
#                    dns_res     - hash with query parsed results            #
##############################################################################
updateDNSlistsDo() {

	local dig_tool="$1"
        local dns_list="$2"
        local dns_domain="$3"
        local dns_proto="$4"
	local dns_port="$5"
	local dns_qtype=''

	echoMsg INFO "Testing [$dns_list] servers"
	
	#Zerro the NEW DNS list. It will be filled with good IP's returned by DNS queries
	DNS_NEW_LIST=''

	#foreach server in the DNS list query it and extract returned list of DNS servers.
        #store it in hash for future comparison with original DNS list
	for dns in $dns_list; do
		echoMsg DEBUG "Querying DNS server $dns $dns_domain $dns_proto $dns_port"

		getDNSserversList "$dig_tool" "$dns" "$dns_domain" "$dns_proto" "$dns_port" "$dns_qtype"
		rc=$?

		[ $rc -eq 1 ] && return 1
	done

	#Now implement logic based on DNS query returnedstatus
	if [ $rc -eq 2 ]; then

		echoMsg WARN "All servers for domain failed to resolve...trying to fall back..."

		#Tested for base domain
		[ "$dns_domain" = "$DNS_DOMAIN_BASE" ] && [ "$dns_port" = "$DNS_PORT_BASE" ] && return 2	

		#Tested for alternate port
		[ "$dns_domain" = "$DNS_DOMAIN_BASE" ] && [ "$dns_port" = "$DNS_PORT_ALT" ] && return 3	

		#Tested for alternate domain
		[ "$dns_domain" = "$DNS_DOMAIN_ALT" ] && [ "$dns_list" = "$DNSLIST" ] && return 4

		#Tested for global servers 
		[ "$dns_domain" = "$DNS_DOMAIN_ALT" ] && [ "$dns_list" = "$DNS_SERVERS_ALT" ]  && return 5
			
		#Tested for local servers
		[ "$dns_domain" = "$DNS_DOMAIN_ALT" ] && [ "$dns_list" = "$DNS_SERVERS_LCL" ] && 
		[ "$dns_proto"  = "$DNS_PROTO" ] && return 6

		#Tested for alternate protocol - UDP
		[ "$dns_proto" = "$DNS_PROTO_ALT" ] && [ "$dns_list" = "$DNS_SERVERS_LCL" ] && return 7
 
		#Unrecognized case
		echoMsg ERROR "Unrecoginized operation configuration encounted"
		return 1
	fi

	#Ok, we have DNS query reults. Compare it to list we start
        #quering with. Hopefully they will be completly similar
	compareDNSlists "$dns_list"
	rc=$?

	[ $rc -eq 2 ] && return 1    #Seem like something went wrong 

	#If list had been found different. Find the quickes to reponse DNS server
        #put it at the beggining of the list. If we fail, abort
	setNewDNSserversList

	#Now see whar compareDNSlists function returned to us If 0 than evrything is ok
	[ $rc -eq 0 ] && return 8	

	#The DNS servers respond ok, but returned DNS list is differnet than the original one
        #go back and re-run operation with new list settings
	return 9
}

####################################################################################
# updateDNSLists :                                                                 #
#                 $DNS_DOMAIN_BASE = dns-servers.threatstop.local                  #
#                 $DNS_DOMAIN_ALT  = dns-servers.threatstop.com                    #
#                 $DNS_SERVERS_ALT = 8.8.8.8 8.8.4.4 208.67.220.220 208.67.222.222 #
#                 $DNS_PROTO       = +tcp                                          #
#                 $DNS_PROTO_ALT   = +notcp                                        #
#                 $DNS_MAX_TRIES   = 3                                             #
####################################################################################
updateDNSLists() {

	local dns_tool="$DIG_TOOL"
	local dns_list="$DNSLIST"
        local dns_dom="$DNS_DOMAIN_BASE"
        local dns_proto="$DNS_PROTO"
	local dns_port="$DNS_PORT"
        local try_num=1;
	local rc

	#Try to get DNS servers list until or success is reached or all options failed
	while [ 1 ]; do

		echoMsg DEBUG "updateDNSlistsDo $dns_tool $dns_list $dns_dom $dns_proto $dns_port"
		updateDNSlistsDo "$dns_tool" "$dns_list" "$dns_dom" "$dns_proto" "$dns_port"
		rc=$?
                [ $rc -eq 1 ] && return 1;

		#All DNS servers in current list failed to respond
		#switch to alternate port
		if [ $rc -eq 2 ]; then
			dns_port=$DNS_PORT_ALT

			echoMsg DEBUG "Switching to alternate port [$dns_port]"
                        echoMsg DEBUG "dns list  - [$dns_list]"
                        echoMsg DEBUG "dns_dom   - [$dns_dom]"
                        echoMsg DEBUG "dns_proto - [$dns_proto]"
			echoMsg DEBUG "dns_proto - [$dns_port]"
                        continue
		fi

		#All DNS servers in current list failed to respond
                #switch to alternate domain
                if [ $rc -eq 3 ]; then
                        dns_dom=$DNS_DOMAIN_ALT
			dns_port=$DNS_PORT_BASE

                        echoMsg DEBUG "Switching to alternate domain [$dns_dom]"
                        echoMsg DEBUG "dns list  - [$dns_list]"
                        echoMsg DEBUG "dns_dom   - [$dns_dom]"
                        echoMsg DEBUG "dns_proto - [$dns_proto]"
			echoMsg DEBUG "dns_proto - [$dns_port]"
                        continue
                fi

		#All DNS servers in current list failied to respond and
                #also alternate domain had been tried. Switch to alt
                #servers list
                if [ $rc -eq 4 ]; then

                        dns_list=$DNS_SERVERS_ALT
                        dns_dom=$DNS_DOMAIN_ALT
			dns_port=$DNS_PORT_BASE

                        echoMsg DEBUG "Switching to alternate global servers list [$dns_list]"
                        echoMsg DEBUG "dns list  - [$dns_list]"
                        echoMsg DEBUG "dns_dom   - [$dns_dom]"
                        echoMsg DEBUG "dns_proto - [$dns_proto]"
			echoMsg DEBUG "dns_proto - [$dns_port]"
			continue
                fi

		#OK, all servers from the GLOBAL servers list had been tried 
                #and failed trying to fallback to nameservers list defined on 
                #the box resolve.conf file
                if [ $rc -eq 5 ]; then

                        readLocalBoxNameserversLst 

			if [ $? -ne 0 ]; then
				#Could not get local defined list of DNS servers.
				#Skipping to another step	
				echoMsg WARN "Can not get local defined DNS servers list ... Skipping to the next stage ..."
			else 
	
        	                dns_dom=$DNS_DOMAIN_ALT
                	        dns_list=$DNS_SERVERS_LCL
				dns_port=$DNS_PORT_BASE

                        	echoMsg DEBUG "Switching localy defined nameservers list"
	                        echoMsg DEBUG "dns list  - [$dns_list]"
        	                echoMsg DEBUG "dns_dom   - [$dns_dom]"
	                        echoMsg DEBUG "dns_proto - [$dns_proto]"
				echoMsg DEBUG "dns_proto - [$dns_port]"
        	                continue
			fi
                fi

		#All checks failed. Last thing to do is to fall back to UDP protocol and
                #re-do whole process again.  
                if [ $rc -eq 6 ]; then 

                        dns_list=$DNSLIST
                        dns_dom=$DNS_DOMAIN_BASE
                        dns_proto=$DNS_PROTO_ALT
			dns_port=$DNS_PORT_BASE

                        echoMsg DEBUG "Switchibg to UDP protocol"
                        echoMsg DEBUG "dns list  - [$dns_list]"
                        echoMsg DEBUG "dns_dom   - [$dns_dom]"
                        echoMsg DEBUG "dns_proto - [$dns_proto]"
			echoMsg DEBUG "dns_proto - [$dns_port]"
                        continue
                fi

		#We have tried all means and failed ... Abort futher execution ...
                if [ $rc -eq 7 ]; then
                        echoMsg ERROR "Failed to retieve updated DNS servers list"
                        return 1;
                fi
	
		#The servers respond ok but returned DNS list is different
                #than original one. Need to re-run previous procedure with
                #new list settings
                if [ $rc -eq 9 ]; then

                        #Max number of tries reached... Abort futher execution ...
                        if [ $try_num -gt $DNS_MAX_TRIES ]; then
                                echoMsg ERROR "Maximun number of tries to resolve DNS servers list had been reached...aborting"
                                return 1;
                        fi

                        #Run again with new list
                        echoMsg DEBUG "Re-running quiery process again with new DNS list... [${try_num}/${DNS_MAX_TRIES}]"
                        dns_list=$DNS_NEW_LIST
                        try_num=$(expr $try_num + 1)
                        continue
                fi

		#Everyting ok and lists are similar break the loop
                if [ $rc -eq 8 ]; then
                        echoMsg DEBUG "DNS list check completed"
                        break
                fi
	done

	#Now if DNS list had been changed and we successfully passed verification process update
        #the threatstop.conf file and $DNSLIST setting with new DNS data
        #if [ "$dns_list" !=  "$DNSLIST" ]; then
        #        DNSLIST=$dns_list;
        #fi

	DNS_PORT=$dns_port

	#Updating config file with new DNS servers list
	updateDNSConfFile
	[ $? -ne 0 ] && return 1

	return 0
}

#updateDNSLists
